@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2024 Rony Shapiro <ronys@pwsafe.org>.
+* Copyright (c) 2003-2025 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -2296,8 +2296,8 @@ void CPWTreeCtrlX::SetFilterState(bool bState)
 {
   m_bTreeFilterActive = bState;
 
-  // Red if filter active, black if not
-  SetTextColor(m_bTreeFilterActive ? RGB(168, 0, 0) : RGB(0, 0, 0));
+  // Red if filter active, default if not
+  SetTextColor(m_bTreeFilterActive ? RGB(168, 0, 0) : ::GetSysColor(COLOR_WINDOWTEXT));
 }
 
 BOOL CPWTreeCtrlX::OnEraseBkgnd(CDC* pDC)
